import java.util.Random;
import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.io.IOException;

public class SortDriver {

  /** Portable new line string. */
  public static final String NEWLINE = System.getProperty("line.separator");
  
  /** Random number seed. */
  static long seed;

  /** Random number generator. */
  static Random random = new Random();

  /** Items to be searched. */
  private Comparable list[];
  
  /** Timer */
  private ElapsedTime timer = new ElapsedTime();

  /** Generate a random 9 digit seed, and print it so that
   *  the example can be rerun exactly. */
  public void setSeed() {
    seed = (System.currentTimeMillis()) % 1000000000;
    System.out.println("Random seed: " + seed);
  }

  /** Sets up a sorted list of distinct Integers of
   *  length n. */
  public void buildList(int n) {

    // Reset the random number generator.
    random.setSeed(seed);
    list = new Comparable[n];
    
    // Generate random integers between
    //  -5n and 5n (so duplicates aren't
    // too common, and for short lists, we
    // can easily read the numbers).
    for (int k = 0; k < n; k++) {
      list[k] = random.nextInt(10*n) - 5*n;
    }
  }
  
  /** Test the search when item is found. */
  public static void main(String[] args) {
    String fileName;
    PrintWriter out = null;
    if ((args != null) && (args.length > 0))
      fileName = args[0];
    else
      fileName = "sortTimes.txt";
    try {
      out = new PrintWriter(new FileOutputStream( fileName ), true);
    } catch (IOException e) {
      System.out.println("Failed to open " + fileName);
      System.out.println(e);
      System.exit(1);
    }
    SortDriver sd =  new SortDriver();
    
    sd.runTest(out);
    out.close();

  }
  
  public void runTest(PrintWriter out) {
    for (int len = 1000; len<=20000; len += 1000) {
      buildList(len);
      System.out.print(""+len);
      out.print(""+len);
      Comparable[] testList = new Comparable[len];

      System.arraycopy(list, 0, testList, 0, len);
      timer.start();
      ListUtil.selectionSort(testList);
      long t = timer.time();
      System.out.format(" %8.3f", t/1000.0);
      out.format(" %8.3f", t/1000.0);
      
      
      System.arraycopy(list, 0, testList, 0, len);
      timer.start();
      ListUtil.bubbleSort(testList);
      t = timer.time();
      System.out.format(" %8.3f", t/1000.0);
      out.format(" %8.3f", t/1000.0);
      
      System.arraycopy(list, 0, testList, 0, len);
      timer.start();
      ListUtil.insertionSort(testList);
      t = timer.time();
      System.out.format(" %8.3f", t/1000.0);
      out.format(" %8.3f", t/1000.0);
      
      System.arraycopy(list, 0, testList, 0, len);
      timer.start();
      ListUtil.mergeSort(testList);
      t = timer.time();
      System.out.format(" %8.3f", t/1000.0);
      out.format(" %8.3f", t/1000.0);

      System.out.println();
      out.println();
    }
  }

}