import junit.framework.TestCase;
import java.util.Random;

/**
 * A JUnit test case class for binary search.
 */
public class SortTest extends TestCase {
  /** For conditional compilation of debug information. */
  public static final boolean DEBUG = false;
  
  /** Random number seed. */
  static long seed;

  /** Random number generator. */
  static Random random = new Random();

  /** Items to be searched. */
  private Comparable list[];
  
  /** Timer */
  private ElapsedTime timer = new ElapsedTime();

  /** Generate a random 9 digit seed, and print it so that
   *  the example can be rerun exactly. */
  public void setSeed() {
    seed = (System.currentTimeMillis()) % 1000000000;
    System.out.println("Random seed: " + seed);
  }

  /** Sets up a sorted list of distinct Integers of
   *  length n. */
  public void buildList(int n) {

    // Reset the random number generator.
    random.setSeed(seed);
     
    list = new Comparable[n];
    
    for (int k = 0; k < n; k++) {
      list[k] = random.nextInt(10*n) - 5*n;
    }
  }
  
  /** Test the search when item is found. */
  public void testSort() {
    setSeed();
    int len = 1000 + random.nextInt(1000);  
    buildList(len);
    
    Comparable[] testList = new Comparable[len];

    System.arraycopy(list, 0, testList, 0, len);
    timer.start();
    ListUtil.selectionSort(testList);
    long t = timer.time();
    System.out.format("Selection Sort Runtime (in secs): %8.3f\n", t/1000.0);
    
    for (int k = 0; k < len-1; k++) {
      if (DEBUG) {
        System.out.print(" "+testList[k]);
        if (k % 5 == 4)
          System.out.println();
      }
      assertTrue("selectionSort: Non-decreasing order: ", 
                 testList[k].compareTo(testList[k+1]) <= 0 );
    }
    if (DEBUG)
      System.out.println();

    System.arraycopy(list, 0, testList, 0, len);
    timer.start();
    ListUtil.bubbleSort(testList);
    t = timer.time();
    System.out.format("Bubble Sort Runtime (in secs): %8.3f\n", t/1000.0);
    for (int k = 0; k < len-1; k++) {
      if (DEBUG) {
        System.out.print(" "+testList[k]);
        if (k % 5 == 4)
          System.out.println();
      }
      assertTrue("bubbleSort: Non-decreasing order: ", 
                 testList[k].compareTo(testList[k+1]) <= 0 );
    }
    if (DEBUG)
      System.out.println();
    
    System.arraycopy(list, 0, testList, 0, len);
    timer.start();
    ListUtil.insertionSort(testList);
    t = timer.time();
    System.out.format("Insertion Sort Runtime (in secs): %8.3f\n", t/1000.0);
    
    for (int k = 0; k < len-1; k++) {
      if (DEBUG) {
        System.out.print(" "+testList[k]);
        if (k % 5 == 4)
          System.out.println();
      }
      assertTrue("insertionSort: Non-decreasing order: ", 
                 testList[k].compareTo(testList[k+1]) <= 0 );
    }
    if (DEBUG)
      System.out.println();

    System.arraycopy(list, 0, testList, 0, len);
    timer.start();
    ListUtil.mergeSort(testList);
    t = timer.time();
    System.out.format("Merge Sort Runtime (in secs): %8.3f\n", t/1000.0);
    
    for (int k = 0; k < len-1; k++) {
      if (DEBUG) {
        System.out.print(" "+testList[k]);
        if (k % 5 == 4)
          System.out.println();
      }
      assertTrue("arraySort: Non-decreasing order: ", 
                 testList[k].compareTo(testList[k+1]) <= 0 );
    }
    if (DEBUG)
      System.out.println();

  }

}
